@@ -1,28 +1,30 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,14 +35,13 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qfsfileengine_p.h"
 #include "qfsfileengine_iterator_p.h"
+#include "qfilesystemengine_p.h"
 #include "qdatetime.h"
 #include "qdiriterator.h"
 #include "qset.h"
@@ -55,6 +56,10 @@
 #include "private/qcore_unix_p.h"
 #endif
 #include <stdio.h>
+#include <stdlib.h>
+#if defined(Q_OS_MAC)
+# include <private/qcore_mac_p.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -115,85 +120,33 @@ void QFSFileEnginePrivate::init()
     openMode = QIODevice::NotOpen;
     fd = -1;
     fh = 0;
+#if defined (Q_OS_SYMBIAN)
+    symbianFilePos = 0;
+#if !defined(QT_SYMBIAN_USE_NATIVE_FILEMAP)
+    fileHandleForMaps = -1;
+#endif
+#endif
     lastIOCommand = IOFlushCommand;
     lastFlushFailed = false;
     closeFileHandle = false;
 #ifdef Q_OS_WIN
     fileAttrib = INVALID_FILE_ATTRIBUTES;
     fileHandle = INVALID_HANDLE_VALUE;
-    mapHandle = INVALID_HANDLE_VALUE;
+    mapHandle = NULL;
+#ifndef Q_OS_WINCE
     cachedFd = -1;
 #endif
-}
-
-/*!
-    \internal
-
-    Returns the canonicalized form of \a path (i.e., with all symlinks
-    resolved, and all redundant path elements removed.
-*/
-QString QFSFileEnginePrivate::canonicalized(const QString &path)
-{
-    if (path.isEmpty())
-        return path;
-
-    QFileInfo fi;
-    const QChar slash(QLatin1Char('/'));
-    QString tmpPath = path;
-    int separatorPos = 0;
-    QSet<QString> nonSymlinks;
-    QSet<QString> known;
-
-    known.insert(path);
-    do {
-#ifdef Q_OS_WIN
-        // UNC, skip past the first two elements
-        if (separatorPos == 0 && tmpPath.startsWith(QLatin1String("//")))
-            separatorPos = tmpPath.indexOf(slash, 2);
-        if (separatorPos != -1)
 #endif
-        separatorPos = tmpPath.indexOf(slash, separatorPos + 1);
-        QString prefix = separatorPos == -1 ? tmpPath : tmpPath.left(separatorPos);
-        if (
-#ifdef Q_OS_SYMBIAN
-            // Symbian doesn't support directory symlinks, so do not check for link unless we
-            // are handling the last path element. This not only slightly improves performance,
-            // but also saves us from lot of unnecessary platform security check failures
-            // when dealing with files under *:/private directories.
-            separatorPos == -1 &&
-#endif
-            !nonSymlinks.contains(prefix)) {
-            fi.setFile(prefix);
-            if (fi.isSymLink()) {
-                QString target = fi.symLinkTarget();
-                if (separatorPos != -1) {
-                    if (fi.isDir() && !target.endsWith(slash))
-                        target.append(slash);
-                    target.append(tmpPath.mid(separatorPos));
-                }
-                tmpPath = QDir::cleanPath(target);
-                separatorPos = 0;
-
-                if (known.contains(tmpPath))
-                    return QString();
-                known.insert(tmpPath);
-            } else {
-                nonSymlinks.insert(prefix);
-            }
-        }
-    } while (separatorPos != -1);
-
-    return QDir::cleanPath(tmpPath);
 }
 
 /*!
     Constructs a QFSFileEngine for the file name \a file.
 */
-QFSFileEngine::QFSFileEngine(const QString &file) : QAbstractFileEngine(*new QFSFileEnginePrivate)
+QFSFileEngine::QFSFileEngine(const QString &file)
+    : QAbstractFileEngine(*new QFSFileEnginePrivate)
 {
     Q_D(QFSFileEngine);
-    d->filePath = QDir::fromNativeSeparators(file);
-    d->nativeInitFileName();
+    d->fileEntry = QFileSystemEntry(file);
 }
 
 /*!
@@ -242,8 +195,7 @@ void QFSFileEngine::setFileName(const QString &file)
 {
     Q_D(QFSFileEngine);
     d->init();
-    d->filePath = QDir::fromNativeSeparators(file);
-    d->nativeInitFileName();
+    d->fileEntry = QFileSystemEntry(file);
 }
 
 /*!
@@ -252,7 +204,7 @@ void QFSFileEngine::setFileName(const QString &file)
 bool QFSFileEngine::open(QIODevice::OpenMode openMode)
 {
     Q_D(QFSFileEngine);
-    if (d->filePath.isEmpty()) {
+    if (d->fileEntry.isEmpty()) {
         qWarning("QFSFileEngine::open: No file name specified");
         setError(QFile::OpenError, QLatin1String("No file name specified"));
         return false;
@@ -280,6 +232,19 @@ bool QFSFileEngine::open(QIODevice::OpenMode openMode)
     success; otherwise returns false.
 */
 bool QFSFileEngine::open(QIODevice::OpenMode openMode, FILE *fh)
+{
+    return open(openMode, fh, QFile::DontCloseHandle);
+}
+
+/*!
+    Opens the file handle \a fh in \a openMode mode. Returns true
+    on success; otherwise returns false.
+
+    The \a handleFlags argument specifies whether the file handle will be
+    closed by Qt. See the QFile::FileHandleFlags documentation for more
+    information.
+*/
+bool QFSFileEngine::open(QIODevice::OpenMode openMode, FILE *fh, QFile::FileHandleFlags handleFlags)
 {
     Q_D(QFSFileEngine);
 
@@ -293,9 +258,8 @@ bool QFSFileEngine::open(QIODevice::OpenMode openMode, FILE *fh)
 
     d->openMode = openMode;
     d->lastFlushFailed = false;
-    d->closeFileHandle = false;
-    d->nativeFilePath.clear();
-    d->filePath.clear();
+    d->closeFileHandle = (handleFlags & QFile::AutoCloseHandle);
+    d->fileEntry.clear();
     d->tried_stat = 0;
     d->fd = -1;
 
@@ -337,6 +301,19 @@ bool QFSFileEnginePrivate::openFh(QIODevice::OpenMode openMode, FILE *fh)
     on success; otherwise returns false.
 */
 bool QFSFileEngine::open(QIODevice::OpenMode openMode, int fd)
+{
+    return open(openMode, fd, QFile::DontCloseHandle);
+}
+
+/*!
+    Opens the file descriptor \a fd in \a openMode mode. Returns true
+    on success; otherwise returns false.
+
+    The \a handleFlags argument specifies whether the file handle will be
+    closed by Qt. See the QFile::FileHandleFlags documentation for more
+    information.
+*/
+bool QFSFileEngine::open(QIODevice::OpenMode openMode, int fd, QFile::FileHandleFlags handleFlags)
 {
     Q_D(QFSFileEngine);
 
@@ -350,9 +327,8 @@ bool QFSFileEngine::open(QIODevice::OpenMode openMode, int fd)
 
     d->openMode = openMode;
     d->lastFlushFailed = false;
-    d->closeFileHandle = false;
-    d->nativeFilePath.clear();
-    d->filePath.clear();
+    d->closeFileHandle = (handleFlags & QFile::AutoCloseHandle);
+    d->fileEntry.clear();
     d->fh = 0;
     d->fd = -1;
     d->tried_stat = 0;
@@ -408,7 +384,14 @@ bool QFSFileEngine::close()
 bool QFSFileEnginePrivate::closeFdFh()
 {
     Q_Q(QFSFileEngine);
-    if (fd == -1 && !fh)
+    if (fd == -1 && !fh
+#ifdef Q_OS_SYMBIAN
+        && !symbianFile.SubSessionHandle()
+#ifndef QT_SYMBIAN_USE_NATIVE_FILEMAP
+        && fileHandleForMaps == -1
+#endif
+#endif
+        )
         return false;
 
     // Flush the file if it's buffered, and if the last flush didn't fail.
@@ -416,10 +399,24 @@ bool QFSFileEnginePrivate::closeFdFh()
     bool closed = true;
     tried_stat = 0;
 
+#if defined(Q_OS_SYMBIAN) && !defined(QT_SYMBIAN_USE_NATIVE_FILEMAP)
+    // Map handle is always owned by us so always close it
+    if (fileHandleForMaps >= 0) {
+        QT_CLOSE(fileHandleForMaps);
+        fileHandleForMaps = -1;
+    }
+#endif
+
     // Close the file if we created the handle.
     if (closeFileHandle) {
         int ret;
         do {
+#ifdef Q_OS_SYMBIAN
+            if (symbianFile.SubSessionHandle()) {
+                symbianFile.Close();
+                ret = 0;
+            } else
+#endif
             if (fh) {
                 // Close buffered file.
                 ret = fclose(fh) != 0 ? -1 : 0;
@@ -496,31 +493,22 @@ qint64 QFSFileEngine::size() const
     return d->nativeSize();
 }
 
+#ifndef Q_OS_WIN
 /*!
     \internal
 */
 qint64 QFSFileEnginePrivate::sizeFdFh() const
 {
     Q_Q(const QFSFileEngine);
-    // ### Fix this function, it should not stat unless the file is closed.
-    QT_STATBUF st;
-    int ret = 0;
     const_cast<QFSFileEngine *>(q)->flush();
-    if (fh && nativeFilePath.isEmpty()) {
-        // Buffered stdlib mode.
-        // ### This should really be an ftell
-        ret = QT_FSTAT(QT_FILENO(fh), &st);
-    } else if (fd == -1) {
-        // Stateless stat.
-        ret = QT_STAT(nativeFilePath.constData(), &st);
-    } else {
-        // Unbuffered stdio mode.
-        ret = QT_FSTAT(fd, &st);
-    }
-    if (ret == -1)
+
+    tried_stat = 0;
+    metaData.clearFlags(QFileSystemMetaData::SizeAttribute);
+    if (!doStat(QFileSystemMetaData::SizeAttribute))
         return 0;
-    return st.st_size;
+    return metaData.size();
 }
+#endif
 
 /*!
     \reimp
@@ -789,6 +777,7 @@ qint64 QFSFileEnginePrivate::writeFdFh(const char *data, qint64 len)
     return writtenBytes;
 }
 
+#ifndef QT_NO_FILESYSTEMITERATOR
 /*!
     \internal
 */
@@ -804,6 +793,7 @@ QAbstractFileEngine::Iterator *QFSFileEngine::endEntryList()
 {
     return 0;
 }
+#endif
 
 /*!
     \internal
@@ -827,18 +817,14 @@ bool QFSFileEngine::isSequential() const
 /*!
     \internal
 */
+#ifdef Q_OS_UNIX
 bool QFSFileEnginePrivate::isSequentialFdFh() const
 {
-    if (!tried_stat)
-        doStat();
-    if (could_stat) {
-#ifdef Q_OS_UNIX
-        return (st.st_mode & S_IFMT) != S_IFREG;
-        // ### WINDOWS!
-#endif
-    }
+    if (doStat(QFileSystemMetaData::SequentialType))
+        return metaData.isSequential();
     return true;
 }
+#endif
 
 /*!
     \reimp
@@ -993,6 +979,10 @@ bool QFSFileEngine::supportsExtension(Extension extension) const
   to store temporary files).
 */
 
+/*! \fn QAbstractFileEngine::FileFlags QFSFileEnginePrivate::getPermissions(QAbstractFileEngine::FileFlags type) const
+    \internal
+*/
+
 QT_END_NAMESPACE
 
 #endif // QT_NO_FSFILEENGINE
