@@ -1,28 +1,30 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,16 +35,12 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "qhelpsearchquerywidget.h"
 
-#include <QtCore/QDebug>
-
 #include <QtCore/QAbstractListModel>
 #include <QtCore/QObject>
 #include <QtCore/QStringList>
@@ -101,8 +99,9 @@ private:
     };
 
     QHelpSearchQueryWidgetPrivate()
-        : QObject(), simpleSearch(true),
-          searchCompleter(new CompleterModel(this), this)
+        : QObject()
+        , simpleSearch(true)
+        , searchCompleter(new CompleterModel(this), this)
     {
         searchButton = 0;
         advancedSearchWidget = 0;
@@ -120,31 +119,20 @@ private:
         // nothing todo
     }
 
-    QString escapeString(const QString &text)
+    void retranslate()
     {
-        QString retValue = text;
-        const QString escape(QLatin1String("\\"));
-        QStringList escapableCharsList;
-        escapableCharsList << QLatin1String("\\") << QLatin1String("+")
-            << QLatin1String("-") << QLatin1String("!") << QLatin1String("(")
-            << QLatin1String(")") << QLatin1String(":") << QLatin1String("^")
-            << QLatin1String("[") << QLatin1String("]") << QLatin1String("{")
-            << QLatin1String("}") << QLatin1String("~");
-
-        // make sure we won't end up with an empty string
-        foreach (const QString escapeChar, escapableCharsList) {
-            if (retValue.contains(escapeChar))
-                retValue.replace(escapeChar, QLatin1String(""));
-        }
-        if (retValue.trimmed().isEmpty())
-            return retValue;
-
-        retValue = text; // now realy escape the string...
-        foreach (const QString escapeChar, escapableCharsList) {
-            if (retValue.contains(escapeChar))
-                retValue.replace(escapeChar, escape + escapeChar);
-        }
-        return retValue;
+        simpleSearchLabel->setText(QHelpSearchQueryWidget::tr("Search for:"));
+        prevQueryButton->setToolTip(QHelpSearchQueryWidget::tr("Previous search"));
+        nextQueryButton->setToolTip(QHelpSearchQueryWidget::tr("Next search"));
+        searchButton->setText(QHelpSearchQueryWidget::tr("Search"));
+#ifdef QT_CLUCENE_SUPPORT
+        advancedSearchLabel->setText(QHelpSearchQueryWidget::tr("Advanced search"));
+        similarLabel->setText(QHelpSearchQueryWidget::tr("words <B>similar</B> to:"));
+        withoutLabel->setText(QHelpSearchQueryWidget::tr("<B>without</B> the words:"));
+        exactLabel->setText(QHelpSearchQueryWidget::tr("with <B>exact phrase</B>:"));
+        allLabel->setText(QHelpSearchQueryWidget::tr("with <B>all</B> of the words:"));
+        atLeastLabel->setText(QHelpSearchQueryWidget::tr("with <B>at least one</B> of the words:"));
+#endif
     }
 
     QStringList buildTermList(const QString query)
@@ -206,8 +194,8 @@ private:
         }
     }
 
-    void nextOrPrevQuery(int maxOrMinIndex, int addend,
-                         QToolButton *thisButton, QToolButton *otherButton)
+    void nextOrPrevQuery(int maxOrMinIndex, int addend, QToolButton *thisButton,
+        QToolButton *otherButton)
     {
         QueryHistory *queryHist;
         QList<QLineEdit *> lineEdits;
@@ -217,7 +205,7 @@ private:
         } else {
             queryHist = &complexQueries;
             lineEdits << allQuery << atLeastQuery << similarQuery
-                    << withoutQuery << exactQuery;
+                << withoutQuery << exactQuery;
         }
         foreach (QLineEdit *lineEdit, lineEdits)
             lineEdit->clear();
@@ -227,32 +215,10 @@ private:
 
         queryHist->curQuery += addend;
         const QList<QHelpSearchQuery> &query =
-                queryHist->queries.at(queryHist->curQuery);
+            queryHist->queries.at(queryHist->curQuery);
         foreach (const QHelpSearchQuery &queryPart, query) {
-            QLineEdit *lineEdit = 0;
-            switch (queryPart.fieldName) {
-            case QHelpSearchQuery::DEFAULT:
-                lineEdit = defaultQuery;
-                break;
-            case QHelpSearchQuery::ALL:
-                lineEdit = allQuery;
-                break;
-            case QHelpSearchQuery::ATLEAST:
-                lineEdit = atLeastQuery;
-                break;
-            case QHelpSearchQuery::FUZZY:
-                lineEdit = similarQuery;
-                break;
-            case QHelpSearchQuery::WITHOUT:
-                lineEdit = withoutQuery;
-                break;
-            case QHelpSearchQuery::PHRASE:
-                lineEdit = exactQuery;
-                break;
-            default:
-                Q_ASSERT(0);
-            }
-            lineEdit->setText(queryPart.wordList.join(" "));
+            if (QLineEdit *lineEdit = lineEditFor(queryPart.fieldName))
+                lineEdit->setText(queryPart.wordList.join(" "));
         }
 
         if (queryHist->curQuery == maxOrMinIndex)
@@ -260,13 +226,34 @@ private:
         otherButton->setEnabled(true);
     }
 
+    QLineEdit* lineEditFor(const QHelpSearchQuery::FieldName &fieldName) const
+    {
+        switch (fieldName) {
+            case QHelpSearchQuery::DEFAULT:
+                return defaultQuery;
+            case QHelpSearchQuery::ALL:
+                return allQuery;
+            case QHelpSearchQuery::ATLEAST:
+                return atLeastQuery;
+            case QHelpSearchQuery::FUZZY:
+                return similarQuery;
+            case QHelpSearchQuery::WITHOUT:
+                return withoutQuery;
+            case QHelpSearchQuery::PHRASE:
+                return exactQuery;
+            default:
+                Q_ASSERT(0);
+        }
+        return 0;
+    }
+
     void enableOrDisableToolButtons()
     {
-        const QueryHistory &queryHist =
-                simpleSearch ? simpleQueries : complexQueries;
+        const QueryHistory &queryHist = simpleSearch ? simpleQueries
+            : complexQueries;
         prevQueryButton->setEnabled(queryHist.curQuery > 0);
-        nextQueryButton->setEnabled(queryHist.curQuery <
-                                    queryHist.queries.size() - 1);
+        nextQueryButton->setEnabled(queryHist.curQuery
+            < queryHist.queries.size() - 1);
     }
 
 private slots:
@@ -290,50 +277,55 @@ private slots:
         QList<QHelpSearchQuery> queryList;
 #if !defined(QT_CLUCENE_SUPPORT)
         queryList.append(QHelpSearchQuery(QHelpSearchQuery::DEFAULT,
-                                          QStringList(defaultQuery->text())));
+            QStringList(defaultQuery->text())));
 
 #else
         if (defaultQuery->isEnabled()) {
             queryList.append(QHelpSearchQuery(QHelpSearchQuery::DEFAULT,
-                                              buildTermList(escapeString(defaultQuery->text()))));
+                buildTermList(defaultQuery->text())));
         } else {
             const QRegExp exp(QLatin1String("\\s+"));
-            QStringList lst = similarQuery->text().split(exp, QString::SkipEmptyParts);
+            QStringList lst = similarQuery->text().split(exp,
+                QString::SkipEmptyParts);
             if (!lst.isEmpty()) {
                 QStringList fuzzy;
-                foreach (const QString term, lst)
-                    fuzzy += buildTermList(escapeString(term));
-                queryList.append(QHelpSearchQuery(QHelpSearchQuery::FUZZY, fuzzy));
+                foreach (const QString &term, lst)
+                    fuzzy += buildTermList(term);
+                queryList.append(QHelpSearchQuery(QHelpSearchQuery::FUZZY,
+                    fuzzy));
             }
 
             lst = withoutQuery->text().split(exp, QString::SkipEmptyParts);
             if (!lst.isEmpty()) {
                 QStringList without;
-                foreach (const QString term, lst)
-                    without.append(escapeString(term));
-                queryList.append(QHelpSearchQuery(QHelpSearchQuery::WITHOUT, without));
+                foreach (const QString &term, lst)
+                    without.append(term);
+                queryList.append(QHelpSearchQuery(QHelpSearchQuery::WITHOUT,
+                    without));
             }
 
             if (!exactQuery->text().isEmpty()) {
                 QString phrase = exactQuery->text().remove(QLatin1Char('\"'));
-                phrase = escapeString(phrase.simplified());
-                queryList.append(QHelpSearchQuery(QHelpSearchQuery::PHRASE, QStringList(phrase)));
+                phrase = phrase.simplified();
+                queryList.append(QHelpSearchQuery(QHelpSearchQuery::PHRASE,
+                    QStringList(phrase)));
             }
 
             lst = allQuery->text().split(exp, QString::SkipEmptyParts);
             if (!lst.isEmpty()) {
                 QStringList all;
-                foreach (const QString term, lst)
-                    all.append(escapeString(term));
+                foreach (const QString &term, lst)
+                    all.append(term);
                 queryList.append(QHelpSearchQuery(QHelpSearchQuery::ALL, all));
             }
 
             lst = atLeastQuery->text().split(exp, QString::SkipEmptyParts);
             if (!lst.isEmpty()) {
                 QStringList atLeast;
-                foreach (const QString term, lst)
-                    atLeast += buildTermList(escapeString(term));
-                queryList.append(QHelpSearchQuery(QHelpSearchQuery::ATLEAST, atLeast));
+                foreach (const QString &term, lst)
+                    atLeast += buildTermList(term);
+                queryList.append(QHelpSearchQuery(QHelpSearchQuery::ATLEAST,
+                    atLeast));
             }
         }
 #endif
@@ -347,8 +339,9 @@ private slots:
 
     void nextQuery()
     {
-        nextOrPrevQuery((simpleSearch ? simpleQueries : complexQueries).queries.size() - 1,
-                        1, nextQueryButton, prevQueryButton);
+        nextOrPrevQuery((simpleSearch ? simpleQueries
+            : complexQueries).queries.size() - 1, 1, nextQueryButton,
+                prevQueryButton);
     }
 
     void prevQuery()
@@ -360,6 +353,13 @@ private:
     friend class QHelpSearchQueryWidget;
 
     bool simpleSearch;
+    QLabel *simpleSearchLabel;
+    QLabel *advancedSearchLabel;
+    QLabel *similarLabel;
+    QLabel *withoutLabel;
+    QLabel *exactLabel;
+    QLabel *allLabel;
+    QLabel *atLeastLabel;
     QPushButton *searchButton;
     QWidget* advancedSearchWidget;
     QToolButton *showHideAdvancedSearchButton;
@@ -392,8 +392,9 @@ private:
     \fn void QHelpSearchQueryWidget::search()
 
     This signal is emitted when a the user has the search button invoked.
-    After reciving the signal you can ask the QHelpSearchQueryWidget for the build list
-    of QHelpSearchQuery's that you may pass to the QHelpSearchEngine's search() function.
+    After reciving the signal you can ask the QHelpSearchQueryWidget for the
+    build list of QHelpSearchQuery's that you may pass to the QHelpSearchEngine's
+    search() function.
 */
 
 /*!
@@ -408,19 +409,17 @@ QHelpSearchQueryWidget::QHelpSearchQueryWidget(QWidget *parent)
     vLayout->setMargin(0);
 
     QHBoxLayout* hBoxLayout = new QHBoxLayout();
-    QLabel *label = new QLabel(tr("Search for:"), this);
+    d->simpleSearchLabel = new QLabel(this);
     d->defaultQuery = new QLineEdit(this);
     d->defaultQuery->setCompleter(&d->searchCompleter);
     d->prevQueryButton = new QToolButton(this);
     d->prevQueryButton->setArrowType(Qt::LeftArrow);
-    d->prevQueryButton->setToolTip(tr("Previous search"));
     d->prevQueryButton->setEnabled(false);
     d->nextQueryButton = new QToolButton(this);
     d->nextQueryButton->setArrowType(Qt::RightArrow);
-    d->nextQueryButton->setToolTip(tr("Next search"));
     d->nextQueryButton->setEnabled(false);
-    d->searchButton = new QPushButton(tr("Search"), this);
-    hBoxLayout->addWidget(label);
+    d->searchButton = new QPushButton(this);
+    hBoxLayout->addWidget(d->simpleSearchLabel);
     hBoxLayout->addWidget(d->defaultQuery);
     hBoxLayout->addWidget(d->prevQueryButton);
     hBoxLayout->addWidget(d->nextQueryButton);
@@ -439,15 +438,15 @@ QHelpSearchQueryWidget::QHelpSearchQueryWidget(QWidget *parent)
     d->showHideAdvancedSearchButton->setText(QLatin1String("+"));
     d->showHideAdvancedSearchButton->setMinimumSize(25, 20);
 
-    label = new QLabel(tr("Advanced search"), this);
+    d->advancedSearchLabel = new QLabel(this);
     QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
-    sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
-    label->setSizePolicy(sizePolicy);
+    sizePolicy.setHeightForWidth(d->advancedSearchLabel->sizePolicy().hasHeightForWidth());
+    d->advancedSearchLabel->setSizePolicy(sizePolicy);
 
     QFrame* hLine = new QFrame(this);
     hLine->setFrameStyle(QFrame::HLine);
     hBoxLayout->addWidget(d->showHideAdvancedSearchButton);
-    hBoxLayout->addWidget(label);
+    hBoxLayout->addWidget(d->advancedSearchLabel);
     hBoxLayout->addWidget(hLine);
 
     vLayout->addLayout(hBoxLayout);
@@ -457,32 +456,32 @@ QHelpSearchQueryWidget::QHelpSearchQueryWidget(QWidget *parent)
     QGridLayout *gLayout = new QGridLayout(d->advancedSearchWidget);
     gLayout->setMargin(0);
 
-    label = new QLabel(tr("words <B>similar</B> to:"), this);
-    gLayout->addWidget(label, 0, 0);
+    d->similarLabel = new QLabel(this);
+    gLayout->addWidget(d->similarLabel, 0, 0);
     d->similarQuery = new QLineEdit(this);
     d->similarQuery->setCompleter(&d->searchCompleter);
     gLayout->addWidget(d->similarQuery, 0, 1);
 
-    label = new QLabel(tr("<B>without</B> the words:"), this);
-    gLayout->addWidget(label, 1, 0);
+    d->withoutLabel = new QLabel(this);
+    gLayout->addWidget(d->withoutLabel, 1, 0);
     d->withoutQuery = new QLineEdit(this);
     d->withoutQuery->setCompleter(&d->searchCompleter);
     gLayout->addWidget(d->withoutQuery, 1, 1);
 
-    label = new QLabel(tr("with <B>exact phrase</B>:"), this);
-    gLayout->addWidget(label, 2, 0);
+    d->exactLabel = new QLabel(this);
+    gLayout->addWidget(d->exactLabel, 2, 0);
     d->exactQuery = new QLineEdit(this);
     d->exactQuery->setCompleter(&d->searchCompleter);
     gLayout->addWidget(d->exactQuery, 2, 1);
 
-    label = new QLabel(tr("with <B>all</B> of the words:"), this);
-    gLayout->addWidget(label, 3, 0);
+    d->allLabel = new QLabel(this);
+    gLayout->addWidget(d->allLabel, 3, 0);
     d->allQuery = new QLineEdit(this);
     d->allQuery->setCompleter(&d->searchCompleter);
     gLayout->addWidget(d->allQuery, 3, 1);
 
-    label = new QLabel(tr("with <B>at least one</B> of the words:"), this);
-    gLayout->addWidget(label, 4, 0);
+    d->atLeastLabel = new QLabel(this);
+    gLayout->addWidget(d->atLeastLabel, 4, 0);
     d->atLeastQuery = new QLineEdit(this);
     d->atLeastQuery->setCompleter(&d->searchCompleter);
     gLayout->addWidget(d->atLeastQuery, 4, 1);
@@ -490,6 +489,8 @@ QHelpSearchQueryWidget::QHelpSearchQueryWidget(QWidget *parent)
     vLayout->addWidget(d->advancedSearchWidget);
     d->advancedSearchWidget->hide();
 
+    d->retranslate();
+
     connect(d->exactQuery, SIGNAL(returnPressed()), this, SIGNAL(search()));
     connect(d->similarQuery, SIGNAL(returnPressed()), this, SIGNAL(search()));
     connect(d->withoutQuery, SIGNAL(returnPressed()), this, SIGNAL(search()));
@@ -510,8 +511,31 @@ QHelpSearchQueryWidget::~QHelpSearchQueryWidget()
 }
 
 /*!
-    Returns a list of querys to use in combination with the search engines
-    search(QList<QHelpSearchQuery> &query) function.
+    \since 4.8
+
+    Expands the search query widget so that the extended search fields are shown.
+*/
+void QHelpSearchQueryWidget::expandExtendedSearch()
+{
+    if (d->simpleSearch)
+        d->showHideAdvancedSearch();
+}
+
+/*!
+    \since 4.8
+
+    Collapses the search query widget so that only the default search field is
+    shown.
+*/
+void QHelpSearchQueryWidget::collapseExtendedSearch()
+{
+    if (!d->simpleSearch)
+        d->showHideAdvancedSearch();
+}
+
+/*!
+    Returns a list of queries to use in combination with the search engines
+    search(QList<QHelpSearchQuery> &queryList) function.
 */
 QList<QHelpSearchQuery> QHelpSearchQueryWidget::query() const
 {
@@ -521,7 +545,32 @@ QList<QHelpSearchQuery> QHelpSearchQueryWidget::query() const
         QList<QHelpSearchQuery>() : queryHist.queries.last();
 }
 
-/*! \reimp
+/*!
+    \since 4.8
+
+    Sets the QHelpSearchQueryWidget input fields to the values specified by
+    \a queryList search field name. Please note that one has to call the search
+    engine's search(QList<QHelpSearchQuery> &queryList) function to perform the
+    actual search.
+*/
+void QHelpSearchQueryWidget::setQuery(const QList<QHelpSearchQuery> &queryList)
+{
+    QList<QLineEdit *> lineEdits;
+    lineEdits << d->defaultQuery << d->allQuery << d->atLeastQuery
+        << d->similarQuery << d->withoutQuery << d->exactQuery;
+    foreach (QLineEdit *lineEdit, lineEdits)
+        lineEdit->clear();
+
+    const QLatin1String space(" ");
+    foreach (const QHelpSearchQuery &q, queryList) {
+        if (QLineEdit *lineEdit = d->lineEditFor(q.fieldName))
+            lineEdit->setText(lineEdit->text() + q.wordList.join(space) + space);
+    }
+    d->searchRequested();
+}
+
+/*!
+    \reimp
 */
 void QHelpSearchQueryWidget::focusInEvent(QFocusEvent *focusEvent)
 {
@@ -531,4 +580,14 @@ void QHelpSearchQueryWidget::focusInEvent(QFocusEvent *focusEvent)
     }
 }
 
+/*! \reimp
+*/
+void QHelpSearchQueryWidget::changeEvent(QEvent *event)
+{
+    if (event->type() == QEvent::LanguageChange)
+        d->retranslate();
+    else
+        QWidget::changeEvent(event);
+}
+
 QT_END_NAMESPACE
